#include "cpu.h"
#include "cpu2.h"
#include "mem.h"

//---------------------------------------------------------------------------
ST_CPU Cpu;


//---------------------------------------------------------------------------
EWRAM_CODE void CpuInit(void)
{
	_Memset(&Cpu, 0x00, sizeof(ST_CPU));

	Cpu.AF.uw = 0x01b0;
	Cpu.BC.uw = 0x0013;
	Cpu.DE.uw = 0x00d8;
	Cpu.HL.uw = 0x014d;
	Cpu.SP.uw = 0xfffe;
	Cpu.PC.uw = 0x0100;
}
//---------------------------------------------------------------------------
EWRAM_CODE void CpuSetGbs(u16 adrLoad, u16 adrInit, u16 adrPlay, u16 sp, u32 songNum)
{
	Cpu.baseRst = adrLoad;
	Cpu.adrInit = adrInit;
	Cpu.adrPlay = adrPlay;
	Cpu.baseSP  = sp;
	Cpu.SP.uw   = sp;
	Cpu.songNum = songNum;
}
//---------------------------------------------------------------------------
IWRAM_CODE void CpuExecInit(void)
{
	CpuPushStack(Cpu.PC.uw);
	Cpu.PC.uw  = Cpu.adrInit;
	Cpu.AF.b.h = Cpu.songNum;

//	TRACEOUT("INIT\n");
//	CpuDebugOn();
	CpuExec();
}
//---------------------------------------------------------------------------
IWRAM_CODE void CpuExecPlay(void)
{
	CpuPushStack(Cpu.PC.uw);
	Cpu.PC.uw  = Cpu.adrPlay;
	Cpu.AF.b.h = 0;

//	TRACEOUT("PLAY\n");
//	CpuDebugOn();
//	for(;;){}
	CpuExec();
}
//---------------------------------------------------------------------------
IWRAM_CODE void CpuPushStack(u16 val)
{
	Cpu.SP.uw -= 2;

	MemWriteWord(Cpu.SP.uw, val);
}
//---------------------------------------------------------------------------
IWRAM_CODE u16 CpuPopStack(void)
{
	u16 ret = MemReadWord(Cpu.SP.uw);
	Cpu.SP.uw += 2;

	return ret;
}
//---------------------------------------------------------------------------
IWRAM_CODE void CpuDebugOn(void)
{
	Cpu.debugFlag = TRUE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void CpuError(void)
{
	SystemError("Unknown opcode:\n[%04X]: AF:%04X BC:%04X DE:%04X HL:%04X SP:%04X  m:%02X m1:%02X m2:%02X\n", 
		Cpu.PC.uw, Cpu.AF.uw, Cpu.BC.uw,
		Cpu.DE.uw, Cpu.HL.uw, Cpu.SP.uw,
		MemReadByte(Cpu.PC.uw), MemReadByte(Cpu.PC.uw+1), MemReadByte(Cpu.PC.uw+2));
}
