#ifndef CPU_H
#define CPU_H
#ifdef __cplusplus
extern "C" {
#endif

#include "lib/gba.h"

//---------------------------------------------------------------------------
#define CPU_FLAG_ZERO					0x80
#define CPU_FLAG_SUB					0x40
#define CPU_FLAG_HALF					0x20
#define CPU_FLAG_CARRY					0x10
#define CPU_FLAG_NONE					0x00


//---------------------------------------------------------------------------
typedef struct {
	void (*pFunc)(void);
} ST_CPU_EXEC_TABLE;

typedef union {
	u16 uw;

	struct {
		u8 l;
		u8 h;
	} b;

} ST_CPU_REG;

typedef struct {
	ST_CPU_REG AF;
	ST_CPU_REG BC;
	ST_CPU_REG DE;
	ST_CPU_REG HL;
	ST_CPU_REG SP;
	ST_CPU_REG PC;

	u16  adrInit;
	u16  adrPlay;
	u16  baseRst;
	u16  baseSP;
	u32  songNum;

	bool debugFlag;

} ST_CPU;


//---------------------------------------------------------------------------
EWRAM_CODE void CpuInit(void);
EWRAM_CODE void CpuSetGbs(u16 adrLoad, u16 adrInit, u16 adrPlay, u16 sp, u32 songNum);
IWRAM_CODE void CpuExecInit(void);
IWRAM_CODE void CpuExecPlay(void);

IWRAM_CODE void CpuPushStack(u16 val);
IWRAM_CODE u16  CpuPopStack(void);

IWRAM_CODE void CpuDebugOn(void);
EWRAM_CODE void CpuError(void);


#ifdef __cplusplus
}
#endif
#endif
