#include "gbs.h"
#include "bg.h"
#include "key.h"
#include "cpu.h"
#include "mem.h"

//---------------------------------------------------------------------------
ST_GBS Gbs;


//---------------------------------------------------------------------------
EWRAM_CODE void GbsInit(void)
{
	_Memset(&Gbs, 0x00, sizeof(ST_GBS));

	Gbs.maxFile = GbfsGetArcCnt();
	GbsSetSong();
}
//---------------------------------------------------------------------------
IWRAM_CODE void GbsExec(void)
{
	if(Gbs.select == 0)
	{
		GbsExecFile();
	}
	else
	{
		GbsExecSong();
	}

	u32 key = KeyGetTrg();

	if(key & KEY_A)
	{
		GbsStop();
		GbsPlay();
	}
	else if(key & KEY_B)
	{
		GbsStop();
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void GbsExecFile(void)
{
	u32 key = KeyGetRep();

	if(key & KEY_DOWN)
	{
		Gbs.select++;
	}
	else if(key & KEY_LEFT && Gbs.fileNum != 0)
	{
		Gbs.fileNum--;
		GbsSetSong();
	}
	else if(key & KEY_RIGHT && Gbs.fileNum+1 < Gbs.maxFile)
	{
		Gbs.fileNum++;
		GbsSetSong();
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void GbsExecSong(void)
{
	u32 key = KeyGetRep();

	if(key & KEY_UP)
	{
		Gbs.select--;
	}
	else if(key & KEY_LEFT && Gbs.songNum != 0)
	{
		Gbs.songNum--;
	}
	else if(key & KEY_RIGHT && Gbs.songNum+1 < Gbs.maxSong)
	{
		Gbs.songNum++;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void GbsDraw(void)
{
	BgDrawStr(0, 0, "GBS Player for GBA");

	BgDrawPrintf(2, 2, "File %03d/%03d", Gbs.fileNum + 1, Gbs.maxFile);
	BgDrawPrintf(2, 3, "Song %03d/%03d", Gbs.songNum + 1, Gbs.maxSong);

	BgDrawStr(1, 2, " ");
	BgDrawStr(1, 3, " ");
	BgDrawStr(1, 2+Gbs.select, ">");

	BgDrawStr(2, 5, "                         ");	// 25
	BgDrawStr(2, 5, GbfsGetFileName2(Gbs.fileNum));
}
//---------------------------------------------------------------------------
IWRAM_CODE void GbsPlay(void)
{
	ST_GBS_HEADER* p = (ST_GBS_HEADER*)GbfsGetSafePointer2(Gbs.fileNum);

	MemLoadGbs((u8*)(p + 1), p->adrLoad);

	CpuSetGbs(p->adrLoad, p->adrInit, p->adrPlay, p->adrStack, Gbs.songNum);
	CpuExecInit();

	GbsSetTiming(p->tac, p->tma);
}
//---------------------------------------------------------------------------
IWRAM_CODE void GbsStop(void)
{
	REG_TM3CNT_H = 0;
	REG_TM3CNT_L = 0;

	CpuInit();
	MemInit();
}
//---------------------------------------------------------------------------
IWRAM_CODE void GbsSetTiming(u8 tmc, u8 tma)
{
	// timer or v-blank
	if(tmc & 0x04)
	{
		u32 t = 0x100 - tma;

		switch(tmc & 0x03)
		{
		case 0x0: t *= 64; break;		// {NbN1024    4096 Hz
		case 0x1: t *=  1; break;		// {NbN  16  262144 Hz
		case 0x2: t *=  4; break;		// {NbN  64   65536 Hz
		case 0x3: t *= 16; break;		// {NbN 256   16384 Hz
		}

		// CPU clock rate 2x ?
		if(tmc & 0x80)
		{
			t /= 8;		// TODO ̊Ŏ܂iOOG
			t *= 3;
		}

//		TRACEOUT("%x %x\n", t, tmc & 0x03);

		REG_TM3CNT_L = 0x10000 - t;
		REG_TM3CNT_H = TIMER_FREQ_PER_64 | TIMER_IRQ | TIMER_START;
	}
	else
	{
		REG_TM3CNT_L = 0x10000 - 274;	// 274 = 280896 clock(59.737 Hz) / 1024
		REG_TM3CNT_H = TIMER_FREQ_PER_1024 | TIMER_IRQ | TIMER_START;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void GbsSetSong(void)
{
	ST_GBS_HEADER* p = (ST_GBS_HEADER*)GbfsGetSafePointer2(Gbs.fileNum);

	Gbs.maxSong = p->numSong;
	Gbs.songNum = 0;
}
