#include "mem.h"
#include "gbs.h"


//---------------------------------------------------------------------------
ST_MEM Mem EWRAM_BSS;


//---------------------------------------------------------------------------
EWRAM_CODE void MemInit(void)
{
	_Memset(&Mem, 0x00, sizeof(ST_MEM));

	MemWriteByte(0xff05, 0x00);
	MemWriteByte(0xff06, 0x00);
	MemWriteByte(0xff07, 0x00);

	// sound 1
	MemWriteByte(0xff10, 0x80);
	MemWriteByte(0xff11, 0xbf);
	MemWriteByte(0xff12, 0xf3);
	MemWriteByte(0xff13, 0x00);
	MemWriteByte(0xff14, 0x00);		// uB{̏l0xbf

	// sound 2
	MemWriteByte(0xff16, 0x3f);
	MemWriteByte(0xff17, 0x00);
	MemWriteByte(0xff18, 0x00);
	MemWriteByte(0xff19, 0x00);		// uB{̏l0xbf

	// sound 3
	MemWriteByte(0xff1a, 0x7f);
	MemWriteByte(0xff1b, 0xff);
	MemWriteByte(0xff1c, 0x9f);
	MemWriteByte(0xff1d, 0x00);
	MemWriteByte(0xff1e, 0x00);		// uB{̏l0xbf

	// sound 4
	MemWriteByte(0xff20, 0xff);
	MemWriteByte(0xff21, 0x00);
	MemWriteByte(0xff22, 0x00);
	MemWriteByte(0xff23, 0x00);		// uB{̏l0xbf

	// Sound Ctl
	MemWriteByte(0xff24, 0x77);
	MemWriteByte(0xff25, 0xf3);
	MemWriteByte(0xff26, 0xf1);

	MemWriteByte(0xff40, 0x91);
	MemWriteByte(0xff41, 0x80);
	MemWriteByte(0xff42, 0x00);
	MemWriteByte(0xff43, 0x00);
	MemWriteByte(0xff45, 0x00);
	MemWriteByte(0xff47, 0xfc);
	MemWriteByte(0xff48, 0xff);
	MemWriteByte(0xff49, 0xff);
	MemWriteByte(0xff4a, 0x00);
	MemWriteByte(0xff4b, 0x00);
	MemWriteByte(0xffff, 0x00);
}
//---------------------------------------------------------------------------
EWRAM_CODE void MemLoadGbs(u8* pCode, u16 adrLoad)
{
	if(adrLoad < 0x4000)
	{
		Mem.pRom1 = pCode + (0x4000 - adrLoad);
		Mem.rom1Offset = 0;
	}
	else
	{
		Mem.pRom1 = pCode;
		Mem.rom1Offset = adrLoad - 0x4000;
	}

	MemLoadRom0(pCode, adrLoad);
	MemLoadRom1(1);
}
//---------------------------------------------------------------------------
EWRAM_CODE void MemLoadRom0(u8* pCode, u16 adrLoad)
{
	u32 i;

	// BANK0ꍇ̓XLbv܂
	for(i=adrLoad; i<0x4000; i++)
	{
		Mem.mem[i] = *pCode++;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void MemLoadRom1(u8 byte)
{
	if(byte == 0)
	{
		byte = 1;
	}

	u8* p = Mem.pRom1 + (0x4000 * (byte - 1));
	u32 i;

	for(i=0x4000+Mem.rom1Offset; i<0x8000; i++)
	{
		Mem.mem[i] = *p++;
	}

	Mem.rom1Num = byte;
}
//---------------------------------------------------------------------------
IWRAM_CODE void MemWriteByte(u16 adr, u8 byte)
{
	switch(adr >> 12)
	{
	case 0x2:
	case 0x3:
//		TRACEOUT("BANK: %x = %x\n", adr, byte);
		if(Mem.rom1Num != byte)
		{
			MemLoadRom1(byte);
		}
		return;

	case 0xa:
	case 0xb:
	case 0xc:
	case 0xd:
	case 0xf:
//		TRACEOUT("W %x %x\n", adr, byte);
		Mem.mem[adr] = byte;

		if((adr >> 8) == 0xff)
		{
			MemWriteIO(adr & 0xff, byte);
		}
		return;

	default:
//		TRACEOUT("ERR: %x = %x\n", adr, byte);
		return;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void MemWriteWord(u16 adr, u16 word)
{
	MemWriteByte(adr, word);
	MemWriteByte(adr + 1, word >> 8);
}
//---------------------------------------------------------------------------
IWRAM_CODE void MemWriteIO(u8 adr8, u8 byte)
{
	vu8* p = (vu8*)REG_BASE;

	switch(adr8)
	{
	// Sound
	case 0x10: p += 0x60; break;	// NR 10 TEhP XEB[v
	case 0x11: p += 0x62; break;	// NR 11 TEhP TEh/g`p^[
	case 0x12: p += 0x63; break;	// NR 12 TEhP Gx[u
	case 0x13: p += 0x64; break;	// NR 13 TEhP giʁj
	case 0x14: p += 0x65; break;	// NR 14 TEhP giʁj
	case 0x16: p += 0x68; break;	// NR 21 TEhQ TEh/g`p^[
	case 0x17: p += 0x69; break;	// NR 22 TEhQ Gx[u
	case 0x18: p += 0x6c; break;	// NR 23 TEhQ giʁj
	case 0x19: p += 0x6d; break;	// NR 24 TEhQ giʁj
	case 0x1a: p += 0x70;			// NR 30 TEhR TEhON/OFF
		byte &= 0x80;
		break;
	case 0x1b: p += 0x72; break;	// NR 31 TEhR TEh
	case 0x1c: p += 0x73;			// NR 32 TEhR o̓x
		byte &= 0x60;
		break;
	case 0x1d: p += 0x74; break;	// NR 33 TEhR giʁj
	case 0x1e: p += 0x75; break;	// NR 34 TEhR giʁj
	case 0x20: p += 0x78; break;	// NR 41 TEhS TEh
	case 0x21: p += 0x79; break;	// NR 42 TEhS Gx[u
	case 0x22: p += 0x7c; break;	// NR 43 TEhS JE^
	case 0x23: p += 0x7d; break;	// NR 44 TEhS PEAI/Jn
	case 0x24: p += 0x80; break;	// NR 50 TEh̃{[/͒[q
	case 0x25: p += 0x81; break;	// NR 51 TEho͒[q̑I
	case 0x26: p += 0x84; break;	// NR 52 TEhON/OFFI

	// Sound Wave RAM
	case 0x30:
	case 0x31:
	case 0x32:
	case 0x33:
	case 0x34:
	case 0x35:
	case 0x36:
	case 0x37:
	case 0x38:
	case 0x39:
	case 0x3a:
	case 0x3b:
	case 0x3c:
	case 0x3d:
	case 0x3e:
	case 0x3f:
		REG_SOUND3CNT_L = 0x0040;	// TODO ɖ悤ɂׂɒǉBǂs
		p += 0x90 + (adr8 - 0x30);
		break;

	default:
		// TRACEOUT("W FF%x %x\n", adr8, byte);
		// EMPTY
		break;
	}

	if(p != (u8*)REG_BASE)
	{
//		TRACEOUT("W FF%x %x\n", adr8, byte);
		*p = byte;
	}
}
//---------------------------------------------------------------------------
/*
IWRAM_CODE u8 MemReadByte(u16 adr)
{
	if(adr >= 0xff00 && adr <= 0xff70)
	{
		TRACEOUT("R %X %X\n", adr, Mem.mem[adr]);
	}

 	return Mem.mem[adr];
}
//---------------------------------------------------------------------------
IWRAM_CODE u16 MemReadWord(u16 adr)
{
	if(adr >= 0xff00 && adr <= 0xff70)
	{
		TRACEOUT("R %X %X\n", adr, (Mem.mem[adr] + (Mem.mem[adr + 1] << 8)));
	}

	return Mem.mem[adr] + (Mem.mem[adr + 1] << 8);
}
*/
